//
//  util.cpp
//  
//
//  Created by Roni Mittelman on 8/15/13.
//
//

#include "util.h"

void util::save_matrix2file(char *filename, std::valarray<float> dataV, int num_of_rows, int num_of_cols){
    int i,j;
    std::ofstream outfile;
    outfile.open(filename);
    outfile << (double)num_of_rows << std::endl;
    outfile << (double)num_of_cols << std::endl;
    for (j=0;j<num_of_cols;j++){
        for (i=0;i<num_of_rows;i++){
            outfile<<(double)dataV[i+num_of_rows*j] << std::endl;
        }
    }
    outfile.close();
};

void util::read_data(char *filename, std::vector<std::valarray<float> > *data, std::valarray<int> *Tvct, int *NVisible){
    std::ifstream infile;
    ////// read MOCAP dataset
    infile.open(filename);
    int i,j,k;
    
    double tmp;
    int num_of_videos;
    infile >> tmp;
    num_of_videos=(int)tmp;
    (*Tvct).resize(num_of_videos);
    int NVisible0=0;
    for (k=0;k<num_of_videos;k++){
        infile >> tmp;
        int T=(int)tmp;
        infile >> tmp;
        NVisible0=(int)tmp;
        std::valarray<float> vct(0.0,NVisible0*T);
        for (i=0;i<NVisible0;i++){
            for (j=0;j<T;j++){
                infile >> tmp;
                vct[i+NVisible0*j]=(float)tmp;
            }
        }
        (*data).push_back(vct);
        (*Tvct)[k]=T;
    }
    (*NVisible)=NVisible0;
    infile.close();
};


void util::select_data_indices(std::vector<std::valarray<float> > *data, std::valarray<int> *Tvct, std::valarray<int> indices){
    std::vector<std::valarray<float> > data2;
    std::valarray<int> Tvct2;
    Tvct2.resize(indices.size());
    for (int i=0;i<(int)indices.size();i++){
        Tvct2[i]=(*Tvct)[indices[i]];
        data2.push_back(*((*data).begin()+indices[i]));
    }
    (*Tvct).resize(indices.size());
    (*data).clear();
    for (int i=0;i<(int)indices.size();i++){
        (*Tvct)[i]=Tvct2[i];
        (*data).push_back(*(data2.begin()+i));
    }
};

void util::create_obs_transform_matrix(int NBlocksSide,int BlockSideLen, std::valarray<float> *T){
    int len=(NBlocksSide*BlockSideLen)*(NBlocksSide*BlockSideLen);
    std::valarray<int> indices(0,len);
    
    for (int i=0;i<len;i++){
        indices[i]=i;
    }
    (*T).resize(len*len);
    (*T)=0;
    int k=0;
    for (int i=0;i<NBlocksSide;i++){
        for (int j=0;j<NBlocksSide;j++){
            for (int n=i*BlockSideLen;n<(i+1)*BlockSideLen;n++){
                for (int m=j*BlockSideLen;m<(j+1)*BlockSideLen;m++){
                    (*T)[(k++)+indices[m+n*(NBlocksSide*BlockSideLen)]*len]=1.0;
                }
            }
        }
    }
};

void util::permute_data(std::vector<std::valarray<float> > *data, std::valarray<int> Tvct, int NVisible, std::valarray<float> T){
    std::vector<std::valarray<float> > data2;
    for (int k=0;k<(int)(*data).size();k++){
        std::valarray<float> vct(NVisible*Tvct[k]);
        for (int i=0;i<NVisible;i++){
            int p=0;
            while (T[i+NVisible*p]==0.0)
            {
                p++;
            }
            for (int j=0;j<Tvct[k];j++){
                vct[i+NVisible*j]=(*((*(data)).begin()+k))[p+NVisible*j];
            }
        }
        data2.push_back(vct);
    }
    (*data).clear();
    for (int k=0;k<(int)data2.size();k++){
        (*data).push_back(*(data2.begin()+k));
    }
};


void util::read_obs_transform_matrix(char *filename, std::valarray<float> *T, std::valarray<int> *dims){
    std::ifstream infile;
    double tmp;
    infile.open(filename);
    infile >> tmp;
    int NVisible=(int)tmp;
    std::valarray<int> indices(0,NVisible);
    for (int i=0;i<NVisible;i++){
        infile >> tmp;
        indices[i]=(int)tmp;
    }
    infile >> tmp;
    int num_of_nodes=(int)tmp;
    (*dims).resize(num_of_nodes);
    for (int i=0;i<num_of_nodes;i++){
        infile >> tmp;
        (*dims)[i]=(int)tmp;
    }
    infile.close();
    
    (*T).resize(NVisible*NVisible);
    (*T)=0;
    for (int i=0;i<NVisible;i++){
        (*T)[i+indices[i]*NVisible]=1.0;
    }
};

void util::read_parms(char *filename, char *parameters_filename, int *type, int *n_CD, int *num_of_nodes, std::valarray<int> *NVisible, std::valarray<int> *NHidden, int *num_of_iterations, float *learning_rate, float *momentum, float *l1_Theta_reg, int *flag_learn_mask, int *flag_causal, int *flag_share_lambda_parm, float *alpha_parm, float *lambda_parm_init, float *delta_parm, float *kappa, char** argv){
    int k=1;
    (*type)=atoi(argv[k++]);
    (*n_CD)=atoi(argv[k++]);
    (*num_of_nodes)=atoi(argv[k++]);
    (*NVisible).resize(*num_of_nodes);
    for (int i=0;i<(*num_of_nodes);i++){
        (*NVisible)[i]=atoi(argv[k++]);
    }
    (*NHidden).resize(*num_of_nodes);
    for (int i=0;i<(*num_of_nodes);i++){
        (*NHidden)[i]=atoi(argv[k++]);
    }
    (*num_of_iterations)=atoi(argv[k++]);
    (*learning_rate)=atof(argv[k++]);
    (*momentum)=atof(argv[k++]);
    (*l1_Theta_reg)=atof(argv[k++]);
    (*flag_learn_mask)=atoi(argv[k++]);
    (*flag_causal)=atoi(argv[k++]);
    (*flag_share_lambda_parm)=atoi(argv[k++]);
    (*alpha_parm)=atof(argv[k++]);
    (*lambda_parm_init)=atof(argv[k++]);
    (*delta_parm)=atof(argv[k++]);
    (*kappa)=atof(argv[k++]);
    strcpy(filename,argv[k++]);
    strcpy(parameters_filename,argv[k]);
};